/// USER STYLESHEET FUNCTIONS

// File mode flags

const MODE_RDONLY   = 0x01;
const MODE_WRONLY   = 0x02;
const MODE_RDWR     = 0x04;
const MODE_CREATE   = 0x08;
const MODE_APPEND   = 0x10;
const MODE_TRUNCATE = 0x20;
const MODE_SYNC     = 0x40;
const MODE_EXCL     = 0x80;


var Flashblock = {

    // Returns a nsIFile for the specified file in the profile chrome directory
    getUserChromeFile : function(fileName) {
        var NSIFILE = Components.interfaces.nsIFile;
        var dirLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                                   .getService(Components.interfaces.nsIProperties);
        var userChromePath = dirLocator.get("UChrm", NSIFILE).path;

        var file = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);

        file.initWithPath(userChromePath);
        file.append(fileName);

        return file;
    },

    // Returns the contents of the specified file in the profile chrome directory
    readUserChromeFile : function (fileName) {
        var fileContents = "";
        var file = Flashblock.getUserChromeFile(fileName);

        if(file.exists()) {
            var ioFlags = MODE_RDONLY;

            // Get an input stream
            var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
                               .createInstance( Components.interfaces.nsIFileInputStream);
            is.init(file, ioFlags, 0, is.CLOSE_ON_EOF);
            var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
                                .createInstance( Components.interfaces.nsIScriptableInputStream);
            sis.init(is);

            // Read the file in
            while(sis.available() > 0)
                fileContents += sis.read(sis.available());

            // Close streams
            is.close();
            sis.close();
        }

        return fileContents;
    },

    // Writes the specified contents into the specified file in the profile chrome directory
    writeUserChromeFile : function (fileName, fileContents) {
        var file = Flashblock.getUserChromeFile(fileName);
        var ioFlags = MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE;
        var perm = 0644;

        var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                           .createInstance( Components.interfaces.nsIFileOutputStream);
        os.init(file, ioFlags, perm, 0);

        var result = os.write(fileContents, fileContents.length);
        os.close();

        return result;
    },

    // Checks the if the user stylesheet already contains the import statement
    userStyleSheetHasImport : function () {
        var fileContents = Flashblock.readUserChromeFile('userContent.css');
        var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css", "m");

        return re.test(fileContents);
    },

    // Adds a CSS import statement for the flashblock stylesheet
    addImportToUserStylesheet : function (fileName) {
        var importStr = "@import url(chrome://flashblock/content/flashblock.css);"

        var fileContents = Flashblock.readUserChromeFile('userContent.css');
        var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css", "m");

        if(re.test(fileContents))
            return true;

        fileContents = importStr + "\n" + fileContents;

        var ret = Flashblock.writeUserChromeFile(fileName, fileContents);
        return (ret == fileContents.length);
    },

    // Removes the CSS import statement for the flashblock stylesheet
    removeImportFromUserStylesheet : function (fileName) {
        var fileContents = Flashblock.readUserChromeFile(fileName);
        var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css.*(\n)?$", "mg");

        if(re.test(fileContents)) {
            fileContents = fileContents.replace(re, '');
            var ret = Flashblock.writeUserChromeFile(fileName, fileContents);
            return (ret == fileContents.length)
        } else {
            return true;
        }
    },


/// PREF FUNCTIONS
	// was flashblockPrefObserver
	prefObserver : {
		observe: function(subject, topic, data) {
			if(data == "flashblock.whitelist")
				Flashblock.loadWhitelist();
			if(data == "flashblock.enabled") {
				gFlashblockEnabled = Flashblock.isEnabled();
				Flashblock.setButtonState(gFlashblockEnabled);
				if ((!gFlashblockEnabled) &&
					("@mozilla.org/content/style-sheet-service;1" in Components.classes)) {
					Flashblock.removeImportFromUserStylesheet('userContent.css');
				}
			}
			if(data == "javascript.enabled") {
				Flashblock.setButtonState(gFlashblockEnabled);
			}
		}
	},

	// was addFlashblockPrefObserver
	addPrefObserver : function () {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefBranch);

		var pbi = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("flashblock.enabled", Flashblock.prefObserver, false);
		pbi.addObserver("flashblock.whitelist", Flashblock.prefObserver, false);
		pbi.addObserver("javascript.enabled", Flashblock.prefObserver, false);
	},


/// WHITELIST FUNCTIONS

	// Loads the whitelist into the global array
	loadWhitelist : function () {
		var flashblockPref = getWhitelistPref();
		gFlashblockWhitelist = new Array();
		if (flashblockPref)
		    gFlashblockWhitelist = flashblockPref.split(",");
	},

	checkWhitelist : function (url) {
		if(!isLocalBlocked()) {
			if(url.protocol == "file:")
				return true;
		}
		for (var i = 0; i < gFlashblockWhitelist.length; i++) {
			// Handle *
			var expr = gFlashblockWhitelist[i];
			expr = expr.replace(/\./g, "\\.");
			expr = expr.replace(/\-/g, "\\-");
			expr = expr.replace(/\*/g, "[A-Za-z0-9_\\-\\.]*")
			expr = "^" + expr + "$";

			var re = new RegExp(expr);
			if(re.test(url.host))
				return true;
		}
		return false;
	},

	checkLoadFlash : function (e) {
		if(! gFlashblockEnabled || Flashblock.checkWhitelist(e.target.location))
			e.preventDefault();
		e.stopPropagation();
	},

	// Gets the hostname from the current URI
	getHost : function () {
		var uri = Components.classes['@mozilla.org/network/standard-url;1']
			.createInstance(Components.interfaces.nsIURI);
		if (gContextMenu) {
			uri.spec = gContextMenu.target.baseURI;
		} else {
			// Work around about: URIs. If we do uri.spec = "about:blank",
			// the URI ends up as about://blank/
			if(content.location.protocol == "about:")
				return undefined;
			uri.spec = content.location;
		}

		// Phil: use asciiHost until we change the pref from char to complex
		var host = uri.asciiHost;
		if (uri.port > 0)
			host += ":" + uri.port;

		return host;
	},

	// Adds the host of the current URI to the whitelist
	addSiteToWhitelist : function () {
		var host = this.getHost();
		if( (!host) || (host == "") )
			return;

		var prefStr = getWhitelistPref();
		var re = new RegExp("(^|,)" + host + "(,|$)");
		if(! re.test(prefStr)) {
			if(prefStr && prefStr.length > 0)
				prefStr += "," + host;
			else
				prefStr = host;
			setWhitelistPref(prefStr);
		}
	},

	// Removes the host of the current URI from the whitelist
	removeSiteFromWhitelist : function () {
		var host = this.getHost();
		if( (!host) || (host == "") )
			return;

		var prefStr = getWhitelistPref();
		var regex = new RegExp("(^|,)(" + host + ")(,|$)", "g");

		prefStr = prefStr.replace(regex, "$3");
		setWhitelistPref(prefStr);
	},

	// Toggles the whitelist status of the host of the current URI
	toggleSiteWhitelisted : function () {
		if(this.checkWhitelist(content.location))
			this.removeSiteFromWhitelist();
		else
			this.addSiteToWhitelist();
	},


/// CONTEXT MENU FUNCTIONS

    contextMenuInit : function () {
        var cm = document.getElementById("contentAreaContextMenu");
        if (cm)
            cm.addEventListener("popupshowing",Flashblock.contextMenu,false);
        Flashblock.setButtonState(isEnabled());
    },

    // was flashblockContextMenu()
    contextMenu : function () {
      var cm = gContextMenu;
      var onFlash = cm.hasBGImage &&
        cm.bgImageURL.indexOf("chrome://flashblock") == 0;

      cm.showItem("context-flashAllow", onFlash);
      cm.showItem("context-flashWhitelist", onFlash);
      // document.getElementById("context-flashRemove").hidden = !onFlash;
      if (onFlash) {
        var thisURI = Components.classes['@mozilla.org/network/standard-url;1']
            .createInstance(Components.interfaces.nsIURI);
        thisURI.spec = gContextMenu.target.baseURI;
        document.getElementById("context-flashAllow")
          .setAttribute("tooltiptext", thisURI.prePath);
      };

      if(onFlash) {
		Flashblock.checkWhitelistToggle();

		// Lor: XXX HACK: isn't there a better way?
		// Phil: Don't know. Anyway changing getElementById() to showItem()
		cm.showItem("context-back", false);
		cm.showItem("context-forward", false);
		cm.showItem("context-reload", false);
		cm.showItem("context-stop", false);

		cm.showItem("context-sep-stop", false);

		cm.showItem("context-bookmarkpage", false);
		cm.showItem("context-savepage", false);
		cm.showItem("context-sendpage", false);
		cm.showItem("context-viewbgimage", false);
		cm.showItem("context-viewbgimage-menu", false);

		cm.showItem("context-sep-viewbgimage", false);

		cm.showItem("context-selectall", false);
		cm.showItem("context-viewsource", false);
		cm.showItem("context-viewinfo", false);
		cm.showItem("context-metadata", false);
		cm.showItem("context-sep-properties", false);

		// Netscape
		cm.showItem("switchToTrident", false);
		cm.showItem("context-print", false);
      }
    },

    // was flashblockOptions()
    showOptions : function() {
        window.openDialog("chrome://flashblock/content/options.xul",
            "FlashblockOptions", "chrome,titlebar,toolbar,centerscreen,resizable");
    },


/// INITIALIZATION CODE
    onInstall : function() {
        window.removeEventListener("load", Flashblock.onInstall, true);

        if("@mozilla.org/content/style-sheet-service;1" in Components.classes) {
          // we have the new stylesheet api!
          if(gFlashblockEnabled && !Flashblock.isFlashblockInstalled()) {
            var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
                      .getService(Components.interfaces.nsIStyleSheetService);
            var ios = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
            var u = ios.newURI("chrome://flashblock/content/flashblock.css", null, null);
            if(!sss.sheetRegistered(u, sss.USER_SHEET)) {
              sss.loadAndRegisterSheet(u, sss.USER_SHEET);
              dump("installing flashblock\n");
            }
          }
          gFlashblockInstalled = true;
        }
        else {
          if(! Flashblock.userStyleSheetHasImport()) {
            Flashblock.addImportToUserStylesheet('userContent.css');
            var flashblockBundle = document.getElementById("bundle_flashblock");
            var brandBundle = document.getElementById("bundle_brand");
            var brandShortName = brandBundle.getString("brandShortName");
            var msg = flashblockBundle.getFormattedString("flashblockInstalled",
                                                          [brandShortName]);
            alert(msg);
          }
        }
    },


/// TOOLBARBUTTON CODE

  //was flashblockToggleButton() {
    toggleButton : function() {
        var state = !gFlashblockEnabled;
        setEnabled(state);
        Flashblock.setButtonState(state);
    },

  //was flashblockSetButtonState(state)
    setButtonState : function(state) {
        var button = document.getElementById("flashblockToggle-button");
        if (button != null) {
            button.setAttribute("state", state? "enabled" : "disabled");
            button.setAttribute("disabled", isJavascriptEnabled()?
                                "false" : "true");
        }
        button = document.getElementById("flashblockMozToggle-button");
        if (button != null) {
            button.setAttribute("state", state? "enabled" : "disabled");
            button.setAttribute("disabled", isJavascriptEnabled()?
                                "false" : "true");
        }
    },

    setTooltipSite : function() {
		var thisURI = Components.classes['@mozilla.org/network/standard-url;1']
			.createInstance(Components.interfaces.nsIURI);
        thisURI.spec = getBrowser().mCurrentBrowser.currentURI.spec;
        var menu = document.getElementById("buttonmenu-flashblockAllow");
        if (menu) { 
            menu.setAttribute("tooltiptext", thisURI.prePath)
        };
    },

    // workaround for bug 147670
    fixParentTip : function(state, pnode) {
        var tip;
        if (state == "hide") {
            tip = pnode.getAttribute("tooltiptext");
            if (tip) {
                pnode.setAttribute("temptip", tip);
                pnode.removeAttribute("tooltiptext");
            };
            Flashblock.setTooltipSite();
        } else if (state == "show") {
            tip = pnode.getAttribute("temptip");
            if (tip) {
                pnode.setAttribute("tooltiptext", tip);
                pnode.removeAttribute("temptip");
            };
        }
    },

	checkWhitelistToggle : function () {
		var host = this.getHost();
		var toolbarWhitelistItem = document.getElementById("buttonmenu-flashblockAllow");
		var contextmenuWhitelistItem = document.getElementById("context-flashAllow");

		if(host) {
			toolbarWhitelistItem.setAttribute("disabled", false);
			contextmenuWhitelistItem.setAttribute("disabled", false);

			var whitelisted = this.checkWhitelist(content.location);
			toolbarWhitelistItem.setAttribute("checked", whitelisted)
			contextmenuWhitelistItem.setAttribute("checked", whitelisted);

		} else {
			toolbarWhitelistItem.setAttribute("disabled", true);
			contextmenuWhitelistItem.setAttribute("disabled", true);
		}
    },

/// MISC

    //was flashblockHideObject()
    hideObject : function() {
        if(gContextMenu) {
            var obj = gContextMenu.target;
            if(obj) {
                obj.style.display = "none";
            }
        }
    },

    // the isEnabled() function in flashblock-prefs.js seems to go out of scope 
    isEnabled : function() {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefBranch);

        if(prefs.getPrefType("flashblock.enabled") == prefs.PREF_BOOL)
            return prefs.getBoolPref("flashblock.enabled");
        else
        	return true;
    },
    
    isFlashblockInstalled : function() {
      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
               .getService(Components.interfaces.nsIWindowMediator);
      var en = wm.getEnumerator("navigator:browser");

      while(en.hasMoreElements()) {
        var w = en.getNext();
        if(w.gFlashblockInstalled)
          return true;
      }
      return false;
    },

    // Window counting
    windowCount : function() {
        var wn = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                 .getService(Components.interfaces.nsIWindowWatcher)
                 .getWindowEnumerator();
        var count = 0;
        while (wn.hasMoreElements()) {
            count++;
            wn.getNext();
        }
        return count;
    }

}

var gFlashblockWhitelist;
var gFlashblockEnabled;
var gFlashblockInstalled = false;

Flashblock.addPrefObserver();
Flashblock.loadWhitelist();
gFlashblockEnabled = Flashblock.isEnabled();

window.addEventListener("load", Flashblock.onInstall, true);
window.addEventListener("load", Flashblock.contextMenuInit, true);
window.addEventListener("flashblockCheckLoad", Flashblock.checkLoadFlash, true, true)
